<?php

namespace Drupal\smart_content\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\smart_content\Variation\VariationInterface;
use Drupal\smart_content\VariationSetType\VariationSetTypeInterface;

//use Drupal\smart_content\VariationInterface;

/**
 * Defines the Smart variation set entity.
 *
 * @ConfigEntityType(
 *   id = "smart_variation_set",
 *   label = @Translation("Smart variation set"),
 *   handlers = {
 *     "list_builder" = "Drupal\smart_content\SmartVariationSetListBuilder",
 *     "form" = {
 *       "add" = "Drupal\smart_content\Form\SmartVariationSetForm",
 *       "edit" = "Drupal\smart_content\Form\SmartVariationSetForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\smart_content\SmartVariationSetHtmlRouteProvider",
 *     },
 *   },
 *   config_prefix = "smart_variation_set",
 *   admin_permission = "administer site configuration",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "uuid",
 *     "variations_settings",
 *     "variation_set_type_settings",
 *     "decision_agent_settings",
 *   },
 *   links = {
 *     "canonical" =
 *   "/admin/structure/smart_variation_set/{smart_variation_set}",
 *   }
 * )
 */
class SmartVariationSet extends ConfigEntityBase implements SmartVariationSetInterface {

  //@todo: determine best way to handle Unlimited Reactions/Variations vs Set Number of Reactions/Variations & Single Reactions/Variations

  /**
   * The Smart variation set ID.
   *
   * @var string
   */
  protected $id;

  /**
   * The Smart variation set label.
   *
   * @var string
   */
  protected $label;

  protected $variations;

  protected $variation_set_type_settings;

  protected $decision_agent_settings;

  protected $additional_settings;

  protected $variationSetTypeInstance;

  protected $decisionAgentInstance;


  /**
   */
  public function getVariationSetType() {
    if (!isset($this->variationSetTypeInstance)) {
      $this->variationSetTypeInstance = $this->getVariationSetTypeFromSettings();
    }
    return $this->variationSetTypeInstance;
  }

  /**
   * @param \Drupal\smart_content\VariationSetType\VariationSetTypeInterface $variation_set_type
   */
  public function setVariationSetType(VariationSetTypeInterface $variation_set_type) {
    $this->variationSetTypeInstance = $variation_set_type;
  }

  /**
   */
  public function addVariation(VariationInterface $variation) {
    if ($variation->id() === NULL) {
      $variation->setId($this->generateUniquePluginId($variation, array_keys($this->getVariations())));
    }
    //@todo: find better way to do this.
    $variation->parentInstance = $this;
    $variation->entity = $this;
    $this->variations[$variation->id()] = $variation;

  }

  /**
   * @return array
   */
  public function getVariations() {
    if (!isset($this->variations)) {
      $this->variations = [];
      foreach ($this->getVariationsFromSettings() as $plugin) {
        //@todo: find better way to do this.
        $plugin->parentInstance = $this;
        $plugin->entity = $this;
        $this->addVariation($plugin);
      }
    }
    return $this->variations;
  }

  /**
   * @param $id
   *
   * @return mixed
   */
  public function getVariation($id) {
    foreach ($this->getVariations() as $variation) {
      if ($variation->id() == $id) {
        return $variation;
      }
    }
    return NULL;
  }

  /**
   * @param $id
   */
  public function removeVariation($id) {
    unset($this->variations[$id]);
  }

  /**
   * Temporary solution for handling variation order until core bug is fixed
   * for tables and weight.
   */
  public function sortVariations() {
    if ($this->getVariations()) {
      uasort($this->variations, function ($first, $second) {
        return $first->getWeight() > $second->getWeight();
      });
    }
  }

  protected function writeChangesToSettings() {
    if ($variation_set_type = $this->getVariationSetType()) {
      $variation_set_type->writeChangesToConfiguration();
      $this->variation_set_type_settings = $variation_set_type->getConfiguration();
    }

    $this->variations_settings = [];
    foreach ($this->getVariations() as $variation) {
      $variation->writeChangesToConfiguration();
      $this->variations_settings[] = $variation->getConfiguration();
    }
  }


  /**
   * {@inheritdoc}
   */
  public function save() {
    $this->writeChangesToSettings();

    if (empty($this->id)) {
      $this->id = \Drupal::service('uuid')->generate();
    }
    parent::save();
  }

  protected function getVariationSetTypeFromSettings() {
    if (!empty($this->variation_set_type_settings)) {
      return \Drupal::getContainer()
        ->get('plugin.manager.smart_content.variation_set_type')
        ->createInstance($this->variation_set_type_settings['plugin_id'], $this->variation_set_type_settings);
    }
    return NULL;
  }

  protected function getVariationsFromSettings() {
    $plugins = [];
    if (!empty($this->variations_settings)) {
      foreach ($this->variations_settings as $id => $value) {
        $plugins[] = \Drupal::getContainer()
          ->get('plugin.manager.smart_content.variation')
          ->createInstance($value['plugin_id'], $value);
      }
    }
    return $plugins;
  }

  public static function generateUniquePluginId($plugin, $existing_ids) {
    $count = 1;
    $machine_default = $plugin->getPluginId();
    while (in_array($machine_default, $existing_ids)) {
      $machine_default = $plugin->getPluginId() . '_' . ++$count;
    }
    return $machine_default;
  }

  /**
   * Get decision agent.
   *
   * @return \Drupal\smart_content\DecisionAgent\DecisionAgentInterface
   * //@todo: refactor with proper configuration handling.
   */
  function getDecisionAgent() {
    if (!isset($this->decisionAgentInstance)) {
      if (!isset($this->decision_agent_settings['plugin_id'])) {
        //@todo: don't set default here
        $this->decision_agent_settings['plugin_id'] = 'client';
      }
      // @todo: add dependency injection
      $this->decisionAgentInstance = \Drupal::service('plugin.manager.smart_content.decision_agent')
        ->createInstance($this->decision_agent_settings['plugin_id']);
      $this->decisionAgentInstance->entity = $this;
    }
    return $this->decisionAgentInstance;
  }

  /**
   * Render placeholder and attach libraries and settings.
   *
   * @return mixed
   */
  public function renderPlaceholder() {
    $content = $this->getDecisionAgent()->renderPlaceholder();
    $content['#attached'] = [
      'library' => $this->getLibraries(),
      'drupalSettings' => [
        'smartContentDecisions' => $this->getAttachedSettings(),
      ],
    ];
    return $content;
  }

  /**
   * Get JS libraries for all child instances.
   *
   * @return array
   */
  public function getLibraries() {
    $libraries = [];
    foreach ($this->getVariations() as $variation) {
      $libraries = array_unique(array_merge($libraries, $variation->getLibraries()));
    }
    $libraries = array_unique(array_merge($libraries, $this->getDecisionAgent()
      ->getLibraries()));
    return $libraries;
  }

  /**
   * Get JS drupalSettings for all child instances.
   *
   * @return array
   */
  public function getAttachedSettings() {
    $settings = [];
    foreach ($this->getVariations() as $variation) {
      $settings[] = $variation->getAttachedSettings();
    }
    return [
      $this->getPlaceholderDecisionId() => [
        'name' => $this->getPlaceholderDecisionId(),
        'agent' => $this->getDecisionAgent()->getPluginId(),
        'variations' => $settings,
      ],
    ];
  }

  /**
   * Get placeholder for attribute.
   *
   * @return string
   */
  function getPlaceholderDecisionId() {
    return $this->getEntityTypeId() . '.' . $this->id();
  }


  public function getVariationResponse($id) {
    if ($variation = $this->getVariation($id)) {
      return $variation->getResponse();
    }
    return NULL;
  }

}
