<?php

namespace Drupal\smart_content\Form;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\smart_content\DecisionAgent\DecisionAgentManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class SmartVariationSetForm.
 *
 * @package Drupal\smart_content\Form
 */
class SmartVariationSetForm extends EntityForm {

  /**
   * @var \Drupal\smart_content\Entity\SmartVariationSet
   */
  protected $entity;

  /**
   * Class constructor.
   */
  public function __construct(DecisionAgentManager $decisionAgentManager) {
    $this->decisionAgentManager = $decisionAgentManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Instantiates this form class.
    return new static(
    // Load the service required to construct this class.
      $container->get('plugin.manager.smart_content.decision_agent')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {

    if ($entity = $form_state->get('variation_set')) {
      $this->entity = $entity;
    }

    $form['#process'][] = [$this, 'buildWidget'];

    return parent::form($form, $form_state);
  }

  /**
   * Render API callback: builds the formatter settings elements.
   */
  public function buildWidget(array &$element, FormStateInterface $form_state, array &$complete_form) {

    if ($variation_set_type = $this->entity->getVariationSetType()) {
      self::pluginForm($variation_set_type, $element, $form_state, ['variation_set_type']);
    }

    $decision_agent_plugins = $this->decisionAgentManager->getDefinitions();

    //@todo: Re-introduce server side decision_agent.
    //    $element['decision_agent'] = [
    //      '#title' => 'Decision Agent',
    //      '#type' => 'radios',
    //      '#default_value' => $this->entity->getDecisionAgent()->getPluginId(),
    //      '#options' => array_combine(array_keys($decision_agent_plugins), array_column($decision_agent_plugins, 'label')),
    //    ];
    $element['decision_agent'] = [
      '#type' => 'value',
      '#value' => 'client',
    ];

    $element['#attached']['library'][] = 'smart_content/form';
    return $element;
  }

  /**
   * Utility function for attaching plugin forms.
   *
   * This function attaches forms for plugins implementing
   * Drupal\Core\Plugin\PluginFormInterface.  The plugin form is automatically
   * provided a Drupal\Core\Form\SubformState for tracking $form_state at the
   * plugin level.
   *
   * @param \Drupal\Component\Plugin\PluginBase $plugin
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @param $parents
   */
  public static function pluginForm($plugin, array &$form, FormStateInterface $form_state, $parents) {
    if ($plugin instanceof PluginFormInterface) {
      if (!$plugin_form = NestedArray::getValue($form, $parents)) {
        $plugin_form = [];
      }
      $plugin_form_state = SubformState::createForSubform($plugin_form, $form, $form_state);
      $plugin_form = $plugin->buildConfigurationForm($plugin_form, $plugin_form_state);
      $plugin_form['#tree'] = TRUE;
      //
      NestedArray::setValue($form, $parents, $plugin_form);
    }
  }

  /**
   * Utility function for validating plugin forms.
   *
   * @param \Drupal\Component\Plugin\PluginBase plugin
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @param array $parents
   */
  public static function pluginFormValidate($plugin, array &$form, FormStateInterface $form_state, array $parents) {
    if ($plugin instanceof PluginFormInterface) {
      if (!$plugin_form = NestedArray::getValue($form, $parents)) {
        $plugin_form = [];
      }
      $plugin_form_state = SubformState::createForSubform($plugin_form, $form, $form_state);
      $plugin->validateConfigurationForm($plugin_form, $plugin_form_state);
    }
  }

  /**
   * Utility function for submitting plugin forms.
   *
   * @param \Drupal\Component\Plugin\PluginBase plugin
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @param array $parents
   */
  public static function pluginFormSubmit($plugin, array &$form, FormStateInterface $form_state, array $parents) {
    if ($plugin instanceof PluginFormInterface) {
      if (!$plugin_form = NestedArray::getValue($form, $parents)) {
        $plugin_form = [];
      }
      $plugin_form_state = SubformState::createForSubform($plugin_form, $form, $form_state);
      $plugin->submitConfigurationForm($plugin_form, $plugin_form_state);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Load form from VariationSetType.
    if ($variation_set_type = $this->entity->getVariationSetType()) {
      self::pluginFormSubmit($variation_set_type, $form, $form_state, ['variation_set_type']);
    }

    $this->save($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {

    $status = $this->entity->save();

    $entity = \Drupal::entityTypeManager()
      ->getStorage('smart_variation_set')->load($this->entity->id());
    switch ($status) {
      case SAVED_NEW:
        drupal_set_message($this->t('Created the %label Variation Set.', [
          '%label' => $this->entity->label(),
        ]));
        break;

      default:
        drupal_set_message($this->t('Saved the %label Variation Set.', [
          '%label' => $this->entity->label(),
        ]));
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    return [];
  }

  public static function saveEntityState($form_state, $parents, $entity) {
    NestedArray::setValue($form_state->getStorage(), array_merge(['variation_sets'], $parents, ['entities']), $entity);
  }

  public static function getEntityState($form_state, $parents) {
    return NestedArray::getValue($form_state->getStorage(), array_merge(['variation_sets'], $parents, ['entities']));
  }

}

